@@ -1,5 +1,6 @@
 // SPDX-License-Identifier: (Apache-2.0 OR MIT)
 #![cfg_attr(feature = "unstable-simd", feature(core_intrinsics))]
+#![allow(internal_features)]
 #![allow(unused_unsafe)]
 #![allow(clippy::missing_safety_doc)]
 #![allow(clippy::redundant_field_names)]
@@ -11,8 +12,8 @@ mod util;
 
 mod deserialize;
 mod exc;
+mod ext;
 mod ffi;
-mod lazy;
 mod opt;
 mod serialize;
 mod typeref;
@@ -29,7 +30,7 @@ use std::ptr::NonNull;
 const PACKB_DOC: &str =
     "packb(obj, /, default=None, option=None)\n--\n\nSerialize Python objects to msgpack.\0";
 const UNPACKB_DOC: &str =
-    "unpackb(obj, /, option=None)\n--\n\nDeserialize msgpack to Python objects.\0";
+    "unpackb(obj, /, ext_hook=None, option=None)\n--\n\nDeserialize msgpack to Python objects.\0";
 
 macro_rules! module_add_object {
     ($mptr: expr, $name: expr, $object:expr) => {
@@ -53,7 +54,7 @@ pub unsafe extern "C" fn PyInit_ormsgpack() -> *mut PyModuleDef {
             ml_meth: PyMethodDefPointer {
                 _PyCFunctionFastWithKeywords: packb,
             },
-            ml_flags: pyo3::ffi::METH_FASTCALL | METH_KEYWORDS,
+            ml_flags: METH_FASTCALL | METH_KEYWORDS,
             ml_doc: PACKB_DOC.as_ptr() as *const c_char,
         },
         PyMethodDef {
@@ -61,7 +62,7 @@ pub unsafe extern "C" fn PyInit_ormsgpack() -> *mut PyModuleDef {
             ml_meth: PyMethodDefPointer {
                 _PyCFunctionFastWithKeywords: unpackb,
             },
-            ml_flags: pyo3::ffi::METH_FASTCALL | METH_KEYWORDS,
+            ml_flags: METH_FASTCALL | METH_KEYWORDS,
             ml_doc: UNPACKB_DOC.as_ptr() as *const c_char,
         },
         PyMethodDef::zeroed(),
@@ -134,6 +135,7 @@ pub unsafe extern "C" fn ormsgpack_exec(mptr: *mut PyObject) -> c_int {
 
     module_add_object!(mptr, "MsgpackDecodeError\0", typeref::MsgpackDecodeError);
     module_add_object!(mptr, "MsgpackEncodeError\0", typeref::MsgpackEncodeError);
+    module_add_object!(mptr, "Ext\0", typeref::EXT_TYPE as *mut PyObject);
 
     0
 }
@@ -172,9 +174,10 @@ pub unsafe extern "C" fn unpackb(
     nargs: Py_ssize_t,
     kwnames: *mut PyObject,
 ) -> *mut PyObject {
+    let mut ext_hook: Option<NonNull<PyObject>> = None;
     let mut optsptr: Option<NonNull<PyObject>> = None;
 
-    let num_args = pyo3::ffi::PyVectorcall_NARGS(nargs as usize);
+    let num_args = PyVectorcall_NARGS(nargs as usize);
     if unlikely!(num_args != 1) {
         let msg = if num_args > 1 {
             Cow::Borrowed("unpackb() accepts only 1 positional argument")
@@ -188,7 +191,9 @@ pub unsafe extern "C" fn unpackb(
         if tuple_size > 0 {
             for i in 0..=tuple_size - 1 {
                 let arg = PyTuple_GET_ITEM(kwnames, i as Py_ssize_t);
-                if arg == typeref::OPTION {
+                if arg == typeref::EXT_HOOK {
+                    ext_hook = Some(NonNull::new_unchecked(*args.offset(num_args + i)));
+                } else if arg == typeref::OPTION {
                     optsptr = Some(NonNull::new_unchecked(*args.offset(num_args + i)));
                 } else {
                     return raise_unpackb_exception(deserialize::DeserializeError::new(
@@ -216,7 +221,7 @@ pub unsafe extern "C" fn unpackb(
         }
     }
 
-    match crate::deserialize::deserialize(*args, optsbits as opt::Opt) {
+    match crate::deserialize::deserialize(*args, ext_hook, optsbits as opt::Opt) {
         Ok(val) => val.as_ptr(),
         Err(err) => raise_unpackb_exception(err),
     }
@@ -232,7 +237,7 @@ pub unsafe extern "C" fn packb(
     let mut default: Option<NonNull<PyObject>> = None;
     let mut optsptr: Option<NonNull<PyObject>> = None;
 
-    let num_args = pyo3::ffi::PyVectorcall_NARGS(nargs as usize);
+    let num_args = PyVectorcall_NARGS(nargs as usize);
     if unlikely!(num_args == 0) {
         return raise_packb_exception(Cow::Borrowed(
             "packb() missing 1 required positional argument: 'obj'",
