@@ -1,4 +1,4 @@
-use crate::typeref::{ARRAY_STRUCT_STR, NUMPY_TYPES};
+use crate::typeref::{load_numpy_types, ARRAY_STRUCT_STR, NUMPY_TYPES};
 use pyo3::ffi::*;
 use serde::ser::{Serialize, SerializeSeq, Serializer};
 use std::os::raw::{c_char, c_int, c_void};
@@ -10,34 +10,38 @@ macro_rules! slice {
 }
 
 pub fn is_numpy_scalar(ob_type: *mut PyTypeObject) -> bool {
-    if unsafe { NUMPY_TYPES.is_none() } {
+    let numpy_types = unsafe { NUMPY_TYPES.get_or_init(load_numpy_types) };
+    if numpy_types.is_none() {
         false
     } else {
-        let scalar_types = unsafe { NUMPY_TYPES.as_ref().unwrap() };
+        let scalar_types = unsafe { numpy_types.unwrap().as_ref() };
         ob_type == scalar_types.float64
             || ob_type == scalar_types.float32
             || ob_type == scalar_types.int64
             || ob_type == scalar_types.int32
+            || ob_type == scalar_types.int16
             || ob_type == scalar_types.int8
             || ob_type == scalar_types.uint64
             || ob_type == scalar_types.uint32
+            || ob_type == scalar_types.uint16
             || ob_type == scalar_types.uint8
             || ob_type == scalar_types.bool_
     }
 }
 
 pub fn is_numpy_array(ob_type: *mut PyTypeObject) -> bool {
-    if unsafe { NUMPY_TYPES.is_none() } {
+    let numpy_types = unsafe { NUMPY_TYPES.get_or_init(load_numpy_types) };
+    if numpy_types.is_none() {
         false
     } else {
-        unsafe { ob_type == NUMPY_TYPES.as_ref().unwrap().array }
+        let scalar_types = unsafe { numpy_types.unwrap().as_ref() };
+        unsafe { ob_type == scalar_types.array }
     }
 }
 
 #[repr(C)]
 pub struct PyCapsule {
-    pub ob_refcnt: Py_ssize_t,
-    pub ob_type: *mut PyTypeObject,
+    pub ob_base: PyObject,
     pub pointer: *mut c_void,
     pub name: *const c_char,
     pub context: *mut c_void,
@@ -65,9 +69,11 @@ pub enum ItemType {
     F32,
     F64,
     I8,
+    I16,
     I32,
     I64,
     U8,
+    U16,
     U32,
     U64,
 }
@@ -79,9 +85,11 @@ impl ItemType {
             (102, 4) => Some(ItemType::F32),
             (102, 8) => Some(ItemType::F64),
             (105, 1) => Some(ItemType::I8),
+            (105, 2) => Some(ItemType::I16),
             (105, 4) => Some(ItemType::I32),
             (105, 8) => Some(ItemType::I64),
             (117, 1) => Some(ItemType::U8),
+            (117, 2) => Some(ItemType::U16),
             (117, 4) => Some(ItemType::U32),
             (117, 8) => Some(ItemType::U64),
             _ => None,
@@ -124,10 +132,14 @@ impl NumpyArray {
         } else {
             let num_dimensions = unsafe { (*array).nd as usize };
             if num_dimensions == 0 {
+                ffi!(Py_DECREF(capsule));
                 return Err(PyArrayError::UnsupportedDataType);
             }
             match ItemType::find(array) {
-                None => Err(PyArrayError::UnsupportedDataType),
+                None => {
+                    ffi!(Py_DECREF(capsule));
+                    Err(PyArrayError::UnsupportedDataType)
+                }
                 Some(kind) => {
                     let mut pyarray = NumpyArray {
                         array: array,
@@ -205,8 +217,8 @@ impl NumpyArray {
 impl Drop for NumpyArray {
     fn drop(&mut self) {
         if self.depth == 0 {
-            ffi!(Py_DECREF(self.array as *mut pyo3::ffi::PyObject));
-            ffi!(Py_DECREF(self.capsule as *mut pyo3::ffi::PyObject));
+            ffi!(Py_DECREF(self.array as *mut PyObject));
+            ffi!(Py_DECREF(self.capsule as *mut PyObject));
         }
     }
 }
@@ -242,6 +254,30 @@ impl Serialize for NumpyArray {
                         seq.serialize_element(&DataTypeF32 { obj: each }).unwrap();
                     }
                 }
+                ItemType::U64 => {
+                    let slice: &[u64] = slice!(data_ptr as *const u64, num_items);
+                    for &each in slice.iter() {
+                        seq.serialize_element(&DataTypeU64 { obj: each }).unwrap();
+                    }
+                }
+                ItemType::U32 => {
+                    let slice: &[u32] = slice!(data_ptr as *const u32, num_items);
+                    for &each in slice.iter() {
+                        seq.serialize_element(&DataTypeU32 { obj: each }).unwrap();
+                    }
+                }
+                ItemType::U16 => {
+                    let slice: &[u16] = slice!(data_ptr as *const u16, num_items);
+                    for &each in slice.iter() {
+                        seq.serialize_element(&DataTypeU16 { obj: each }).unwrap();
+                    }
+                }
+                ItemType::U8 => {
+                    let slice: &[u8] = slice!(data_ptr as *const u8, num_items);
+                    for &each in slice.iter() {
+                        seq.serialize_element(&DataTypeU8 { obj: each }).unwrap();
+                    }
+                }
                 ItemType::I64 => {
                     let slice: &[i64] = slice!(data_ptr as *const i64, num_items);
                     for &each in slice.iter() {
@@ -254,34 +290,22 @@ impl Serialize for NumpyArray {
                         seq.serialize_element(&DataTypeI32 { obj: each }).unwrap();
                     }
                 }
+                ItemType::I16 => {
+                    let slice: &[i16] = slice!(data_ptr as *const i16, num_items);
+                    for &each in slice.iter() {
+                        seq.serialize_element(&DataTypeI16 { obj: each }).unwrap();
+                    }
+                }
                 ItemType::I8 => {
                     let slice: &[i8] = slice!(data_ptr as *const i8, num_items);
                     for &each in slice.iter() {
                         seq.serialize_element(&DataTypeI8 { obj: each }).unwrap();
                     }
                 }
-                ItemType::U8 => {
-                    let slice: &[u8] = slice!(data_ptr as *const u8, num_items);
-                    for &each in slice.iter() {
-                        seq.serialize_element(&DataTypeU8 { obj: each }).unwrap();
-                    }
-                }
-                ItemType::U32 => {
-                    let slice: &[u32] = slice!(data_ptr as *const u32, num_items);
-                    for &each in slice.iter() {
-                        seq.serialize_element(&DataTypeU32 { obj: each }).unwrap();
-                    }
-                }
-                ItemType::U64 => {
-                    let slice: &[u64] = slice!(data_ptr as *const u64, num_items);
-                    for &each in slice.iter() {
-                        seq.serialize_element(&DataTypeU64 { obj: each }).unwrap();
-                    }
-                }
                 ItemType::BOOL => {
                     let slice: &[u8] = slice!(data_ptr as *const u8, num_items);
                     for &each in slice.iter() {
-                        seq.serialize_element(&DataTypeBOOL { obj: each }).unwrap();
+                        seq.serialize_element(&DataTypeBool { obj: each }).unwrap();
                     }
                 }
             }
@@ -290,20 +314,6 @@ impl Serialize for NumpyArray {
     }
 }
 
-#[repr(transparent)]
-struct DataTypeF32 {
-    pub obj: f32,
-}
-
-impl Serialize for DataTypeF32 {
-    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
-    where
-        S: Serializer,
-    {
-        serializer.serialize_f32(self.obj)
-    }
-}
-
 #[repr(transparent)]
 pub struct DataTypeF64 {
     pub obj: f64,
@@ -319,72 +329,16 @@ impl Serialize for DataTypeF64 {
 }
 
 #[repr(transparent)]
-pub struct DataTypeI8 {
-    pub obj: i8,
+struct DataTypeF32 {
+    pub obj: f32,
 }
 
-impl Serialize for DataTypeI8 {
+impl Serialize for DataTypeF32 {
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
     {
-        serializer.serialize_i8(self.obj)
-    }
-}
-
-#[repr(transparent)]
-pub struct DataTypeI32 {
-    pub obj: i32,
-}
-
-impl Serialize for DataTypeI32 {
-    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
-    where
-        S: Serializer,
-    {
-        serializer.serialize_i32(self.obj)
-    }
-}
-
-#[repr(transparent)]
-pub struct DataTypeI64 {
-    pub obj: i64,
-}
-
-impl Serialize for DataTypeI64 {
-    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
-    where
-        S: Serializer,
-    {
-        serializer.serialize_i64(self.obj)
-    }
-}
-
-#[repr(transparent)]
-pub struct DataTypeU8 {
-    pub obj: u8,
-}
-
-impl Serialize for DataTypeU8 {
-    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
-    where
-        S: Serializer,
-    {
-        serializer.serialize_u8(self.obj)
-    }
-}
-
-#[repr(transparent)]
-pub struct DataTypeU32 {
-    pub obj: u32,
-}
-
-impl Serialize for DataTypeU32 {
-    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
-    where
-        S: Serializer,
-    {
-        serializer.serialize_u32(self.obj)
+        serializer.serialize_f32(self.obj)
     }
 }
 
@@ -403,11 +357,109 @@ impl Serialize for DataTypeU64 {
 }
 
 #[repr(transparent)]
-pub struct DataTypeBOOL {
+pub struct DataTypeU32 {
+    pub obj: u32,
+}
+
+impl Serialize for DataTypeU32 {
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: Serializer,
+    {
+        serializer.serialize_u32(self.obj)
+    }
+}
+
+#[repr(transparent)]
+pub struct DataTypeU16 {
+    pub obj: u16,
+}
+
+impl Serialize for DataTypeU16 {
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: Serializer,
+    {
+        serializer.serialize_u16(self.obj)
+    }
+}
+
+#[repr(transparent)]
+pub struct DataTypeU8 {
     pub obj: u8,
 }
 
-impl Serialize for DataTypeBOOL {
+impl Serialize for DataTypeU8 {
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: Serializer,
+    {
+        serializer.serialize_u8(self.obj)
+    }
+}
+
+#[repr(transparent)]
+pub struct DataTypeI64 {
+    pub obj: i64,
+}
+
+impl Serialize for DataTypeI64 {
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: Serializer,
+    {
+        serializer.serialize_i64(self.obj)
+    }
+}
+
+#[repr(transparent)]
+pub struct DataTypeI32 {
+    pub obj: i32,
+}
+
+impl Serialize for DataTypeI32 {
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: Serializer,
+    {
+        serializer.serialize_i32(self.obj)
+    }
+}
+
+#[repr(transparent)]
+pub struct DataTypeI16 {
+    pub obj: i16,
+}
+
+impl Serialize for DataTypeI16 {
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: Serializer,
+    {
+        serializer.serialize_i16(self.obj)
+    }
+}
+
+#[repr(transparent)]
+pub struct DataTypeI8 {
+    pub obj: i8,
+}
+
+impl Serialize for DataTypeI8 {
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: Serializer,
+    {
+        serializer.serialize_i8(self.obj)
+    }
+}
+
+#[repr(transparent)]
+pub struct DataTypeBool {
+    pub obj: u8,
+}
+
+impl Serialize for DataTypeBool {
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
@@ -418,7 +470,7 @@ impl Serialize for DataTypeBOOL {
 
 #[repr(transparent)]
 pub struct NumpyScalar {
-    pub ptr: *mut pyo3::ffi::PyObject,
+    pub ptr: *mut PyObject,
 }
 
 impl NumpyScalar {
@@ -434,7 +486,8 @@ impl Serialize for NumpyScalar {
     {
         unsafe {
             let ob_type = ob_type!(self.ptr);
-            let scalar_types = NUMPY_TYPES.as_ref().unwrap();
+            let scalar_types =
+                unsafe { NUMPY_TYPES.get_or_init(load_numpy_types).unwrap().as_ref() };
             if ob_type == scalar_types.float64 {
                 (*(self.ptr as *mut NumpyFloat64)).serialize(serializer)
             } else if ob_type == scalar_types.float32 {
@@ -443,12 +496,16 @@ impl Serialize for NumpyScalar {
                 (*(self.ptr as *mut NumpyInt64)).serialize(serializer)
             } else if ob_type == scalar_types.int32 {
                 (*(self.ptr as *mut NumpyInt32)).serialize(serializer)
+            } else if ob_type == scalar_types.int16 {
+                (*(self.ptr as *mut NumpyInt16)).serialize(serializer)
             } else if ob_type == scalar_types.int8 {
                 (*(self.ptr as *mut NumpyInt8)).serialize(serializer)
             } else if ob_type == scalar_types.uint64 {
                 (*(self.ptr as *mut NumpyUint64)).serialize(serializer)
             } else if ob_type == scalar_types.uint32 {
                 (*(self.ptr as *mut NumpyUint32)).serialize(serializer)
+            } else if ob_type == scalar_types.uint16 {
+                (*(self.ptr as *mut NumpyUint16)).serialize(serializer)
             } else if ob_type == scalar_types.uint8 {
                 (*(self.ptr as *mut NumpyUint8)).serialize(serializer)
             } else if ob_type == scalar_types.bool_ {
@@ -462,8 +519,7 @@ impl Serialize for NumpyScalar {
 
 #[repr(C)]
 pub struct NumpyInt8 {
-    pub ob_refcnt: Py_ssize_t,
-    pub ob_type: *mut PyTypeObject,
+    pub ob_base: PyObject,
     pub value: i8,
 }
 
@@ -476,10 +532,24 @@ impl Serialize for NumpyInt8 {
     }
 }
 
+#[repr(C)]
+pub struct NumpyInt16 {
+    pub ob_base: PyObject,
+    pub value: i16,
+}
+
+impl Serialize for NumpyInt16 {
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: Serializer,
+    {
+        serializer.serialize_i16(self.value)
+    }
+}
+
 #[repr(C)]
 pub struct NumpyInt32 {
-    pub ob_refcnt: Py_ssize_t,
-    pub ob_type: *mut PyTypeObject,
+    pub ob_base: PyObject,
     pub value: i32,
 }
 
@@ -494,8 +564,7 @@ impl Serialize for NumpyInt32 {
 
 #[repr(C)]
 pub struct NumpyInt64 {
-    pub ob_refcnt: Py_ssize_t,
-    pub ob_type: *mut PyTypeObject,
+    pub ob_base: PyObject,
     pub value: i64,
 }
 
@@ -510,8 +579,7 @@ impl Serialize for NumpyInt64 {
 
 #[repr(C)]
 pub struct NumpyUint8 {
-    pub ob_refcnt: Py_ssize_t,
-    pub ob_type: *mut PyTypeObject,
+    pub ob_base: PyObject,
     pub value: u8,
 }
 
@@ -524,10 +592,24 @@ impl Serialize for NumpyUint8 {
     }
 }
 
+#[repr(C)]
+pub struct NumpyUint16 {
+    pub ob_base: PyObject,
+    pub value: u16,
+}
+
+impl Serialize for NumpyUint16 {
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: Serializer,
+    {
+        serializer.serialize_u16(self.value)
+    }
+}
+
 #[repr(C)]
 pub struct NumpyUint32 {
-    pub ob_refcnt: Py_ssize_t,
-    pub ob_type: *mut PyTypeObject,
+    pub ob_base: PyObject,
     pub value: u32,
 }
 
@@ -542,8 +624,7 @@ impl Serialize for NumpyUint32 {
 
 #[repr(C)]
 pub struct NumpyUint64 {
-    pub ob_refcnt: Py_ssize_t,
-    pub ob_type: *mut PyTypeObject,
+    pub ob_base: PyObject,
     pub value: u64,
 }
 
@@ -558,8 +639,7 @@ impl Serialize for NumpyUint64 {
 
 #[repr(C)]
 pub struct NumpyFloat32 {
-    pub ob_refcnt: Py_ssize_t,
-    pub ob_type: *mut PyTypeObject,
+    pub ob_base: PyObject,
     pub value: f32,
 }
 
@@ -574,8 +654,7 @@ impl Serialize for NumpyFloat32 {
 
 #[repr(C)]
 pub struct NumpyFloat64 {
-    pub ob_refcnt: Py_ssize_t,
-    pub ob_type: *mut PyTypeObject,
+    pub ob_base: PyObject,
     pub value: f64,
 }
 
@@ -590,8 +669,7 @@ impl Serialize for NumpyFloat64 {
 
 #[repr(C)]
 pub struct NumpyBool {
-    pub ob_refcnt: Py_ssize_t,
-    pub ob_type: *mut PyTypeObject,
+    pub ob_base: PyObject,
     pub value: bool,
 }
 
